@@ -2,11 +2,20 @@ import datetime
 import decimal
 import time
 import itertools
-import iso8601
 import pprint
 import re
 import translationstring
 
+from .compat import (
+    text_,
+    text_type,
+    string_types,
+    xrange,
+    is_nonstr_iter,
+    )
+
+from . import iso8601
+
 _ = translationstring.TranslationStringFactory('colander')
 
 required = object()
@@ -17,6 +26,9 @@ class _null(object):
     def __nonzero__(self):
         return False
 
+    # py3 compat
+    __bool__ = __nonzero__
+
     def __repr__(self):
         return '<colander.null>'
 
@@ -64,7 +76,7 @@ class Invalid(Exception):
         ``msg`` attribute is iterable, it is returned.  If it is not
         iterable, a single-element list containing the ``msg`` value
         is returned."""
-        if hasattr(self.msg, '__iter__'):
+        if is_nonstr_iter(self.msg):
             return self.msg
         return [self.msg]
 
@@ -169,7 +181,7 @@ class All(object):
         for validator in self.validators:
             try:
                 validator(node, value)
-            except Invalid, e:
+            except Invalid as e:
                 msgs.append(e.msg)
 
         if msgs:
@@ -209,7 +221,7 @@ class Function(object):
         result = self.function(value)
         if not result:
             raise Invalid(node, self.message)
-        if isinstance(result, basestring):
+        if isinstance(result, string_types):
             raise Invalid(node, result)
 
 class Regex(object):
@@ -229,7 +241,7 @@ class Regex(object):
         raised with the ``msg`` error message.
     """
     def __init__(self, regex, msg=None):
-        if isinstance(regex, basestring):
+        if isinstance(regex, string_types):
             self.match_object = re.compile(regex)
         else:
             self.match_object = regex
@@ -251,7 +263,7 @@ class Email(Regex):
         if msg is None:
             msg = _("Invalid email address")
         super(Email, self).__init__(
-            u'(?i)^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}$', msg=msg)
+            text_('(?i)^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}$'), msg=msg)
 
 class Range(object):
     """ Validator which succeeds if the value it is passed is greater
@@ -420,7 +432,7 @@ class Mapping(SchemaType):
     def _validate(self, node, value):
         try:
             return dict(value)
-        except Exception, e:
+        except Exception as e:
             raise Invalid(node,
                           _('"${val}" is not a mapping type: ${err}',
                           mapping = {'val':value, 'err':e})
@@ -437,7 +449,7 @@ class Mapping(SchemaType):
             subval = value.pop(name, null)
             try:
                 result[name] = callback(subnode, subval)
-            except Invalid, e:
+            except Invalid as e:
                 if error is None:
                     error = Invalid(node)
                 error.add(e, num)
@@ -563,7 +575,7 @@ class Tuple(Positional, SchemaType):
             subval = value[num]
             try:
                 result.append(callback(subnode, subval))
-            except Invalid, e:
+            except Invalid as e:
                 if error is None:
                     error = Invalid(node)
                 error.add(e, num)
@@ -698,7 +710,7 @@ class Sequence(Positional, SchemaType):
         for num, subval in enumerate(value):
             try:
                 result.append(callback(node.children[0], subval))
-            except Invalid, e:
+            except Invalid as e:
                 if error is None:
                     error = Invalid(node)
                 error.add(e, num)
@@ -885,19 +897,16 @@ class String(SchemaType):
             return null
 
         try:
-            if isinstance(appstruct, unicode):
-                if self.encoding:
-                    result = appstruct.encode(self.encoding)
-                else:
-                    result = appstruct
-            else:
+            if isinstance(appstruct, (text_type, bytes)):
                 encoding = self.encoding
                 if encoding:
-                    result = unicode(appstruct, encoding).encode(encoding)
+                    result = text_(appstruct, encoding).encode(encoding)
                 else:
-                    result = unicode(appstruct)
+                    result = text_type(appstruct)
+            else:
+                result = text_type(appstruct)
             return result
-        except Exception, e:
+        except Exception as e:
             raise Invalid(node,
                           _('"${val} cannot be serialized: ${err}',
                             mapping={'val':appstruct, 'err':e})
@@ -908,12 +917,14 @@ class String(SchemaType):
 
         try:
             result = cstruct
-            if not isinstance(result, unicode):
+            if isinstance(result, (text_type, bytes)):
                 if self.encoding:
-                    result = unicode(str(cstruct), self.encoding)
+                    result = text_(cstruct, self.encoding)
                 else:
-                    result = unicode(cstruct)
-        except Exception, e:
+                    result = text_type(cstruct)
+            else:
+                result = text_type(cstruct)
+        except Exception as e:
             raise Invalid(node,
                           _('${val} is not a string: %{err}',
                             mapping={'val':cstruct, 'err':e}))
@@ -1146,7 +1157,7 @@ class GlobalObject(SchemaType):
         if not cstruct:
             return null
 
-        if not isinstance(cstruct, basestring):
+        if not isinstance(cstruct, string_types):
             raise Invalid(node,
                           _('"${val}" is not a string',
                             mapping={'val':cstruct}))
@@ -1208,7 +1219,7 @@ class DateTime(SchemaType):
 
     def __init__(self, default_tzinfo=_marker):
         if default_tzinfo is _marker:
-            default_tzinfo = iso8601.iso8601.Utc()
+            default_tzinfo = iso8601.Utc()
         self.default_tzinfo = default_tzinfo
 
     def serialize(self, node, appstruct):
@@ -1235,12 +1246,12 @@ class DateTime(SchemaType):
         try:
             result = iso8601.parse_date(
                 cstruct, default_timezone=self.default_tzinfo)
-        except (iso8601.ParseError, TypeError), e:
+        except (iso8601.ParseError, TypeError) as e:
             try:
                 year, month, day = map(int, cstruct.split('-', 2))
                 result = datetime.datetime(year, month, day,
                                            tzinfo=self.default_tzinfo)
-            except Exception, e:
+            except Exception as e:
                 raise Invalid(node, _(self.err_template,
                                       mapping={'val':cstruct, 'err':e}))
         return result
@@ -1312,7 +1323,7 @@ class Date(SchemaType):
             try:
                 year, month, day = map(int, cstruct.split('-', 2))
                 result = datetime.date(year, month, day)
-            except Exception, e:
+            except Exception as e:
                 raise Invalid(node,
                               _(self.err_template,
                                 mapping={'val':cstruct, 'err':e})
@@ -1390,7 +1401,7 @@ class Time(SchemaType):
             except ValueError:
                 try:
                     result = timeparse(cstruct, '%H:%M')
-                except Exception, e:
+                except Exception as e:
                     raise Invalid(node,
                                   _(self.err_template,
                                     mapping={'val':cstruct, 'err':e})
@@ -1470,7 +1481,7 @@ class SchemaNode(object):
 
     def __new__(cls, *arg, **kw):
         inst = object.__new__(cls)
-        inst._order = cls._counter.next()
+        inst._order = next(cls._counter)
         return inst
 
     def __init__(self, typ, *children, **kw):
@@ -1689,40 +1700,41 @@ class _SchemaMeta(type):
         extended.sort()
         cls.nodes = [x[1] for x in extended]
 
-class Schema(object):
-    schema_type = Mapping
-    node_type = SchemaNode
-    __metaclass__ = _SchemaMeta
+def _Schema__new__(cls, *args, **kw):
+    node = object.__new__(cls.node_type)
+    node.name = None
+    node._order = next(SchemaNode._counter)
+    typ = cls.schema_type()
+    node.__init__(typ, *args, **kw)
+    for n in cls.nodes:
+        node.add(n)
+    return node
 
-    def __new__(cls, *args, **kw):
-        node = object.__new__(cls.node_type)
-        node.name = None
-        node._order = SchemaNode._counter.next()
-        typ = cls.schema_type()
-        node.__init__(typ, *args, **kw)
-        for n in cls.nodes:
-            node.add(n)
-        return node
+Schema = _SchemaMeta('Schema', (object,), 
+    dict(schema_type=Mapping,
+        node_type=SchemaNode,
+        __new__=_Schema__new__))
 
 MappingSchema = Schema
 
-class SequenceSchema(object):
-    schema_type = Sequence
-    node_type = SchemaNode
-    __metaclass__ = _SchemaMeta
 
-    def __new__(cls, *args, **kw):
-        node = object.__new__(cls.node_type)
-        node.name = None
-        node._order = SchemaNode._counter.next()
-        typ = cls.schema_type()
-        node.__init__(typ, *args, **kw)
-        if not len(cls.nodes) == 1:
-            raise Invalid(node,
-                          'Sequence schemas must have exactly one child node')
-        for n in cls.nodes:
-            node.add(n)
-        return node
+def _SequanceSchema__new__(cls, *args, **kw):
+    node = object.__new__(cls.node_type)
+    node.name = None
+    node._order = next(SchemaNode._counter)
+    typ = cls.schema_type()
+    node.__init__(typ, *args, **kw)
+    if not len(cls.nodes) == 1:
+        raise Invalid(node,
+                      'Sequence schemas must have exactly one child node')
+    for n in cls.nodes:
+        node.add(n)
+    return node
+
+SequenceSchema = _SchemaMeta('SequenceSchema', (object,),
+    dict(schema_type=Sequence,
+        node_type=SchemaNode,
+        __new__=_SequanceSchema__new__))
 
 class TupleSchema(Schema):
     schema_type = Tuple
